@@ -1,9 +1,11 @@
+from sqlite3 import DataError
 import numpy as np
 import httpx
-import aiohttp
+from aiohttp_retry import ExponentialRetry, RetryClient
 
 from qiniu import put_file, put_data, Auth, etag
 from qiniu import BucketManager, build_batch_delete
+from async_cow import AsyncCow, ClientCow
 
 from hamunafs.backends.base import BackendBase
 from aiohttp_retry import ExponentialRetry, RetryClient
@@ -16,6 +18,8 @@ class Qiniu(BackendBase):
         self.domain = domain
         self.default_bucket = default_bucket
         self.bucket = BucketManager(self.auth)
+
+        self.cow = AsyncCow(cfg['key'], cfg['secret'])
     
     def get_token(self, filename):
         return self.auth.upload_token(self.default_bucket, filename)
@@ -38,6 +42,21 @@ class Qiniu(BackendBase):
             return False, '上传失败'
         except Exception as e:
             return False, str(e)
+    
+    async def put_async(self, file, bucket, bucket_name, tmp=True):
+        try:
+            if tmp:
+                _bucket = 'tmp_file_' + bucket
+            else:
+                _bucket = bucket
+            b_name = '{}_{}'.format(_bucket, bucket_name)
+            bucket = await self.cow.get_bucket(self.default_bucket)
+            ret, info = await bucket.put_data(key=b_name, file_path=file)
+            if ret is not None:
+                return True, '{}/{}'.format(_bucket, bucket_name)
+            return False, '上传失败'
+        except Exception as e:
+            return False, str(e)
 
     def put_buffer(self, buffer, bucket, bucket_name):
         try:
@@ -50,6 +69,17 @@ class Qiniu(BackendBase):
         except Exception as e:
             return False, str(e)
 
+    async def put_buffer_async(self, buffer, bucket, bucket_name):
+        try:
+            b_name = '{}_{}'.format(bucket, bucket_name)
+            bucket = await self.cow.get_bucket(self.default_bucket)
+            ret, info = await bucket.put_data(key=b_name, data=buffer)
+            if ret is not None:
+                return True, '{}/{}'.format(bucket, bucket_name)
+            return False, '上传失败'
+        except Exception as e:
+            return False, str(e)
+
     def get(self, download_path, bucket, bucket_name, tries=0):
         try:
             if tries >= 3:
@@ -57,10 +87,13 @@ class Qiniu(BackendBase):
             else:
                 url = 'http://{}/{}'.format(self.domain, '{}_{}'.format(bucket, bucket_name))
                 print(url)
-                with open(download_path, mode='wb') as f:
-                    with httpx.stream('GET', url) as response:
-                        for chunk in response.iter_bytes():
-                            f.write(chunk)
+                with httpx.stream('GET', url) as response:
+                    if response.status_code == 200:
+                        with open(download_path, mode='wb') as f:
+                            for chunk in response.iter_bytes():
+                                f.write(chunk)
+                    else:
+                        raise DataError()
                 return True, download_path
         except Exception as e:
             if tries >= 3:
@@ -69,20 +102,24 @@ class Qiniu(BackendBase):
                 return self.get(download_path, bucket, bucket_name, tries+1)
 
     async def get_async(self, download_path, bucket, bucket_name):
-        url = 'http://{}/{}'.format(self.domain, '{}_{}'.format(bucket, bucket_name))
-
         try:
+            url = 'http://{}/{}'.format(self.domain, '{}_{}'.format(bucket, bucket_name))
+            print(url)
             retry_opts = ExponentialRetry(attempts=3)
             async with RetryClient(retry_options=retry_opts) as session:
-                async with session.get(url) as r:
-                    async with AIOFile(download_path, 'wb') as afp:
-                        writer = Writer(afp)
-                        result = await r.read()
-                        await writer(result)
-                        await afp.fsync()
-
-                        return True, download_path
+                async with session.get(url) as response:
+                    if response.status == 200:
+                        with open(download_path, mode='wb') as f:
+                            while True:
+                                chunk = await response.content.read(4096)
+                                if not chunk:
+                                    break
+                                f.write(chunk)
+                    else:
+                        raise DataError()
+            return True, download_path
         except Exception as e:
             return False, str(e)
-        
+
+
             
